#include "common.h"
#include "DispMessage.h"
#include "adapIntgrtn.h"

using namespace System;
using namespace CheckTwoFacetsGen;
using namespace adapInt;

namespace cmmn {


	double * Y;
	int * X;
	double *** XTable;
	double igm_aA, igm_bA, igm_aB, igm_bB, igm_aC, igm_bC,
			igm_aAB, igm_bAB, igm_aAC, igm_bAC, igm_aBC, igm_bBC, igm_aE, igm_bE;
	double * samples_sgmA2, * samples_sgmB2, * samples_sgmC2, 
			* samples_sgmAB2, * samples_sgmAC2, * samples_sgmBC2, * samples_sgmE2, * samples_Rho;
	double *** rho2t_nino, ** rho2t_LB, ** rho2t_UB, ** rho2t_Med;
	

	int N, NA, NB, NC;
	int MaxNi, MaxNo;

	double LogProb( int na, int nb, int nc, double *** xtable, double mu, 
					double * paramA, double * paramB, double * paramC,
					double ** paramAB, double ** paramAC, double ** paramBC,
					double sgmA2, double sgmB2, double sgmC2, 
					double sgmAB2, double sgmAC2, double sgmBC2, double sgmE2 ){
		double v = 0.0;
		v = -(igm_aA + 1.0) * Math::Log(sgmA2) - (igm_bA / sgmA2)
			-(igm_aB + 1.0) * Math::Log(sgmB2) - (igm_bB / sgmB2)
			-(igm_aC + 1.0) * Math::Log(sgmC2) - (igm_bC / sgmC2)
			-(igm_aAB + 1.0) * Math::Log(sgmAB2) - (igm_bAB / sgmAB2)
			-(igm_aAC + 1.0) * Math::Log(sgmAC2) - (igm_bAC / sgmAC2)
			-(igm_aBC + 1.0) * Math::Log(sgmBC2) - (igm_bBC / sgmBC2)
			-(igm_aE + 1.0) * Math::Log(sgmE2) - (igm_bE / sgmE2);
		double sgmA = Math::Sqrt(sgmA2);
		double sgmB = Math::Sqrt(sgmB2);
		double sgmC = Math::Sqrt(sgmC2);
		double sgmAB = Math::Sqrt(sgmAB2);
		double sgmAC = Math::Sqrt(sgmAC2);
		double sgmBC = Math::Sqrt(sgmBC2);
		double sgmE = Math::Sqrt(sgmE2);
		for (int t = 0; t < na; t++)
			v += -Math::Log(sgmA) - 0.5 * sqr(paramA[t] / sgmA);
		for (int i = 0; i < nb; i++)
			v += -Math::Log(sgmB) - 0.5 * sqr(paramB[i] / sgmB);
		for (int j = 0; j < nc; j++)
			v += -Math::Log(sgmC) - 0.5 * sqr(paramC[j] / sgmC);
		for (int t = 0; t < na; t++){
			for (int i = 0; i < nb; i++)
				v += -Math::Log(sgmAB) - 0.5 * sqr(paramAB[t][i] / sgmAB);
			for (int j = 0; j < nc; j++)
				v += -Math::Log(sgmAC) - 0.5 * sqr(paramAC[t][j] / sgmAC);
		}
		for (int i = 0; i < nb; i++)
			for (int j = 0; j < nc; j++)
				v += -Math::Log(sgmBC) - 0.5 * sqr(paramBC[i][j] / sgmBC);
		for (int t = 0; t < na; t++)
			for (int i = 0; i < nb; i++)
				for (int j = 0; j < nc; j++)
					v += -Math::Log(sgmE)
							- 0.5 * sqr((xtable[t][i][j] - (mu + paramA[t] + paramB[i] + paramC[j]
																+ paramAB[t][i] + paramAC[t][j] + paramBC[i][j])) / sgmE);

		return v;
	}


	double LogProb_mu(int na, int nb, int nc, double *** xtable, double mu,
		double * paramA, double * paramB, double * paramC,
		double ** paramAB, double ** paramAC, double ** paramBC,
		double sgmA2, double sgmB2, double sgmC2,
		double sgmAB2, double sgmAC2, double sgmBC2, double sgmE2){
		double v = 0.0;
	
		double sgmE = Math::Sqrt(sgmE2);
		
		for (int t = 0; t < na; t++)
			for (int i = 0; i < nb; i++)
				for (int j = 0; j < nc; j++)
					v += -Math::Log(sgmE)
					- 0.5 * sqr((xtable[t][i][j] - (mu + paramA[t] + paramB[i] + paramC[j]
					+ paramAB[t][i] + paramAC[t][j] + paramBC[i][j])) / sgmE);

		return v;
	}



	double LogProb_A(int na, int nb, int nc, double *** xtable, double mu,
		double * paramA, double * paramB, double * paramC,
		double ** paramAB, double ** paramAC, double ** paramBC,
		double sgmA2, double sgmB2, double sgmC2,
		double sgmAB2, double sgmAC2, double sgmBC2, double sgmE2){
		double v = 0.0;

		double sgmA = Math::Sqrt(sgmA2);

		double sgmE = Math::Sqrt(sgmE2);
		for (int t = 0; t < na; t++)
			v += -Math::Log(sgmA) - 0.5 * sqr(paramA[t] / sgmA);

		for (int t = 0; t < na; t++)
			for (int i = 0; i < nb; i++)
				for (int j = 0; j < nc; j++)
					v += -Math::Log(sgmE)
					- 0.5 * sqr((xtable[t][i][j] - (mu + paramA[t] + paramB[i] + paramC[j]
					+ paramAB[t][i] + paramAC[t][j] + paramBC[i][j])) / sgmE);

		return v;
	}


	double LogProb_B(int na, int nb, int nc, double *** xtable, double mu,
		double * paramA, double * paramB, double * paramC,
		double ** paramAB, double ** paramAC, double ** paramBC,
		double sgmA2, double sgmB2, double sgmC2,
		double sgmAB2, double sgmAC2, double sgmBC2, double sgmE2){
		double v = 0.0;

		double sgmB = Math::Sqrt(sgmB2);

		double sgmE = Math::Sqrt(sgmE2);

		for (int i = 0; i < nb; i++)
			v += -Math::Log(sgmB) - 0.5 * sqr(paramB[i] / sgmB);

		for (int t = 0; t < na; t++)
			for (int i = 0; i < nb; i++)
				for (int j = 0; j < nc; j++)
					v += -Math::Log(sgmE)
					- 0.5 * sqr((xtable[t][i][j] - (mu + paramA[t] + paramB[i] + paramC[j]
					+ paramAB[t][i] + paramAC[t][j] + paramBC[i][j])) / sgmE);

		return v;
	}

	double LogProb_C(int na, int nb, int nc, double *** xtable, double mu,
		double * paramA, double * paramB, double * paramC,
		double ** paramAB, double ** paramAC, double ** paramBC,
		double sgmA2, double sgmB2, double sgmC2,
		double sgmAB2, double sgmAC2, double sgmBC2, double sgmE2){
		double v = 0.0;

		double sgmC = Math::Sqrt(sgmC2);

		double sgmE = Math::Sqrt(sgmE2);

		for (int j = 0; j < nc; j++)
			v += -Math::Log(sgmC) - 0.5 * sqr(paramC[j] / sgmC);

		for (int t = 0; t < na; t++)
			for (int i = 0; i < nb; i++)
				for (int j = 0; j < nc; j++)
					v += -Math::Log(sgmE)
					- 0.5 * sqr((xtable[t][i][j] - (mu + paramA[t] + paramB[i] + paramC[j]
					+ paramAB[t][i] + paramAC[t][j] + paramBC[i][j])) / sgmE);

		return v;
	}

	double LogProb_AB(int na, int nb, int nc, double *** xtable, double mu,
		double * paramA, double * paramB, double * paramC,
		double ** paramAB, double ** paramAC, double ** paramBC,
		double sgmA2, double sgmB2, double sgmC2,
		double sgmAB2, double sgmAC2, double sgmBC2, double sgmE2){
		double v = 0.0;

		double sgmAB = Math::Sqrt(sgmAB2);

		double sgmE = Math::Sqrt(sgmE2);

		for (int t = 0; t < na; t++){
			for (int i = 0; i < nb; i++)
				v += -Math::Log(sgmAB) - 0.5 * sqr(paramAB[t][i] / sgmAB);
		}

		for (int t = 0; t < na; t++)
			for (int i = 0; i < nb; i++)
				for (int j = 0; j < nc; j++)
					v += -Math::Log(sgmE)
					- 0.5 * sqr((xtable[t][i][j] - (mu + paramA[t] + paramB[i] + paramC[j]
					+ paramAB[t][i] + paramAC[t][j] + paramBC[i][j])) / sgmE);

		return v;
	}


	double LogProb_AC(int na, int nb, int nc, double *** xtable, double mu,
		double * paramA, double * paramB, double * paramC,
		double ** paramAB, double ** paramAC, double ** paramBC,
		double sgmA2, double sgmB2, double sgmC2,
		double sgmAB2, double sgmAC2, double sgmBC2, double sgmE2){
		double v = 0.0;

		double sgmAC = Math::Sqrt(sgmAC2);

		double sgmE = Math::Sqrt(sgmE2);

		for (int t = 0; t < na; t++){
			for (int j = 0; j < nc; j++)
				v += -Math::Log(sgmAC) - 0.5 * sqr(paramAC[t][j] / sgmAC);
		}

		for (int t = 0; t < na; t++)
			for (int i = 0; i < nb; i++)
				for (int j = 0; j < nc; j++)
					v += -Math::Log(sgmE)
					- 0.5 * sqr((xtable[t][i][j] - (mu + paramA[t] + paramB[i] + paramC[j]
					+ paramAB[t][i] + paramAC[t][j] + paramBC[i][j])) / sgmE);

		return v;
	}


	double LogProb_BC(int na, int nb, int nc, double *** xtable, double mu,
		double * paramA, double * paramB, double * paramC,
		double ** paramAB, double ** paramAC, double ** paramBC,
		double sgmA2, double sgmB2, double sgmC2,
		double sgmAB2, double sgmAC2, double sgmBC2, double sgmE2){
		double v = 0.0;

		double sgmBC = Math::Sqrt(sgmBC2);
		double sgmE = Math::Sqrt(sgmE2);

		for (int i = 0; i < nb; i++)
			for (int j = 0; j < nc; j++)
				v += -Math::Log(sgmBC) - 0.5 * sqr(paramBC[i][j] / sgmBC);
		for (int t = 0; t < na; t++)
			for (int i = 0; i < nb; i++)
				for (int j = 0; j < nc; j++)
					v += -Math::Log(sgmE)
					- 0.5 * sqr((xtable[t][i][j] - (mu + paramA[t] + paramB[i] + paramC[j]
					+ paramAB[t][i] + paramAC[t][j] + paramBC[i][j])) / sgmE);

		return v;
	}

	double LogProb_sgmA(int na, int nb, int nc, double *** xtable, double mu,
		double * paramA, double * paramB, double * paramC,
		double ** paramAB, double ** paramAC, double ** paramBC,
		double sgmA2, double sgmB2, double sgmC2,
		double sgmAB2, double sgmAC2, double sgmBC2, double sgmE2){
		double v = 0.0;
		v = -(igm_aA + 1.0) * Math::Log(sgmA2) - (igm_bA / sgmA2);

		double sgmA = Math::Sqrt(sgmA2);

		for (int t = 0; t < na; t++)
			v += -Math::Log(sgmA) - 0.5 * sqr(paramA[t] / sgmA);
		
		return v;
	}

	double LogProb_sgmB(int na, int nb, int nc, double *** xtable, double mu,
		double * paramA, double * paramB, double * paramC,
		double ** paramAB, double ** paramAC, double ** paramBC,
		double sgmA2, double sgmB2, double sgmC2,
		double sgmAB2, double sgmAC2, double sgmBC2, double sgmE2){
		double v = 0.0;
		v = -(igm_aB + 1.0) * Math::Log(sgmB2) - (igm_bB / sgmB2);

		double sgmB = Math::Sqrt(sgmB2);

		for (int i = 0; i < nb; i++)
			v += -Math::Log(sgmB) - 0.5 * sqr(paramB[i] / sgmB);

		return v;
	}

	double LogProb_sgmC(int na, int nb, int nc, double *** xtable, double mu,
		double * paramA, double * paramB, double * paramC,
		double ** paramAB, double ** paramAC, double ** paramBC,
		double sgmA2, double sgmB2, double sgmC2,
		double sgmAB2, double sgmAC2, double sgmBC2, double sgmE2){
		double v = 0.0;
		v = -(igm_aC + 1.0) * Math::Log(sgmC2) - (igm_bC / sgmC2);

		double sgmC = Math::Sqrt(sgmC2);

		for (int j = 0; j < nc; j++)
			v += -Math::Log(sgmC) - 0.5 * sqr(paramC[j] / sgmC);

		return v;
	}

	double LogProb_sgmAB(int na, int nb, int nc, double *** xtable, double mu,
		double * paramA, double * paramB, double * paramC,
		double ** paramAB, double ** paramAC, double ** paramBC,
		double sgmA2, double sgmB2, double sgmC2,
		double sgmAB2, double sgmAC2, double sgmBC2, double sgmE2){
		double v = 0.0;
		v = -(igm_aAB + 1.0) * Math::Log(sgmAB2) - (igm_bAB / sgmAB2);

		double sgmAB = Math::Sqrt(sgmAB2);

		for (int t = 0; t < na; t++){
			for (int i = 0; i < nb; i++)
				v += -Math::Log(sgmAB) - 0.5 * sqr(paramAB[t][i] / sgmAB);
		}

		return v;
	}

	double LogProb_sgmAC(int na, int nb, int nc, double *** xtable, double mu,
		double * paramA, double * paramB, double * paramC,
		double ** paramAB, double ** paramAC, double ** paramBC,
		double sgmA2, double sgmB2, double sgmC2,
		double sgmAB2, double sgmAC2, double sgmBC2, double sgmE2){
		double v = 0.0;
		v = -(igm_aAC + 1.0) * Math::Log(sgmAC2) - (igm_bAC / sgmAC2);

		double sgmAC = Math::Sqrt(sgmAC2);

		for (int t = 0; t < na; t++){
			for (int j = 0; j < nc; j++)
				v += -Math::Log(sgmAC) - 0.5 * sqr(paramAC[t][j] / sgmAC);
		}

		return v;
	}

	double LogProb_sgmBC(int na, int nb, int nc, double *** xtable, double mu,
		double * paramA, double * paramB, double * paramC,
		double ** paramAB, double ** paramAC, double ** paramBC,
		double sgmA2, double sgmB2, double sgmC2,
		double sgmAB2, double sgmAC2, double sgmBC2, double sgmE2){
		double v = 0.0;
		v = -(igm_aBC + 1.0) * Math::Log(sgmBC2) - (igm_bBC / sgmBC2);

		double sgmBC = Math::Sqrt(sgmBC2);

		for (int i = 0; i < nb; i++)
			for (int j = 0; j < nc; j++)
				v += -Math::Log(sgmBC) - 0.5 * sqr(paramBC[i][j] / sgmBC);

		return v;
	}


	double LogProb_sgmE(int na, int nb, int nc, double *** xtable, double mu,
		double * paramA, double * paramB, double * paramC,
		double ** paramAB, double ** paramAC, double ** paramBC,
		double sgmA2, double sgmB2, double sgmC2,
		double sgmAB2, double sgmAC2, double sgmBC2, double sgmE2){
		double v = 0.0;
		v = - (igm_aE + 1.0) * Math::Log(sgmE2) - (igm_bE / sgmE2);

		double sgmE = Math::Sqrt(sgmE2);

		for (int t = 0; t < na; t++)
			for (int i = 0; i < nb; i++)
				for (int j = 0; j < nc; j++)
					v += -Math::Log(sgmE)
					- 0.5 * sqr((xtable[t][i][j] - (mu + paramA[t] + paramB[i] + paramC[j]
					+ paramAB[t][i] + paramAC[t][j] + paramBC[i][j])) / sgmE);

		return v;
	}



	
	void MCMC::Init(String ^ msg, int nJmpRN, int vNSimu, int vna, int vnb, int vnc,
						double *** vxtable, double vmu, double * vparamA, double * vparamB, double * vparamC,
						double ** vparamAB, double ** vparamAC, double ** vparamBC,
						double vsgmA2, double vsgmB2, double vsgmC2, double vsgmAB2, double vsgmAC2, double vsgmBC2,
						double vsgmE2, double vgen_SgmMu, double * vgen_paramA, double * vgen_paramB, double * vgen_paramC,
						double ** vgen_paramAB, double ** vgen_paramAC, double ** vgen_paramBC,
						double vgen_sgmA2, double vgen_sgmB2, double vgen_sgmC2, 
						double vgen_sgmAB2, double vgen_sgmAC2, double vgen_sgmBC2, double vgen_sgmE2){
					MnMssg = msg;
					NSimu = vNSimu;
					rn = new RNP2to191();
					for (int i = 0; i < nJmpRN; i++)
						rn->Jump2toj(100);
					NA = vna;
					NB = vnb;
					NC = vnc;
					XTable = new double**[NA];
					for (int t = 0; t < NA; t++){
						XTable[t] = new double*[NB];
						for (int i = 0; i < NB; i++){
							XTable[t][i] = new double[NC];
							for (int j = 0; j < NC; j++)
								XTable[t][i][j] = vxtable[t][i][j];
						}
					}
					mu = new double[NSimu+1];
					mu[0] = vmu;
					paramA = new double*[NSimu+1];
					for (int j = 0; j <= NSimu; j++)
						paramA[j] = new double[NA];
					for (int t = 0; t < NA; t++)
						paramA[0][t] = vparamA[t];
					paramB = new double*[NSimu+1];
					for (int j = 0; j <= NSimu; j++)
						paramB[j] = new double[NB];
					for (int i = 0; i < NB; i++)
						paramB[0][i] = vparamB[i];
					paramC = new double*[NSimu+1];
					for (int i = 0; i <= NSimu; i++)
						paramC[i] = new double[NC];
					for (int i = 0; i < NC; i++)
						paramC[0][i] = vparamC[i];
					paramAB = new double**[NSimu+1];
					for (int i = 0; i <= NSimu; i++){
						paramAB[i] = new double*[NA];
						for (int j = 0; j < NA; j++)
							paramAB[i][j] = new double[NB];
					}
					for (int i = 0; i < NA; i++)
						for (int j = 0; j < NB; j++){
							paramAB[0][i][j] = vparamAB[i][j];		
						}
					paramAC = new double**[NSimu+1];
					for (int i = 0; i <= NSimu; i++){
						paramAC[i] = new double*[NA];
						for (int j = 0; j < NA; j++)
							paramAC[i][j] = new double[NC];
					}
					for (int i = 0; i < NA; i++)
						for (int j = 0; j < NC; j++)
							paramAC[0][i][j] = vparamAC[i][j];
					paramBC = new double**[NSimu+1];
					for (int i = 0; i <= NSimu; i++){
						paramBC[i] = new double*[NB];
						for (int j = 0; j < NB; j++)
							paramBC[i][j] = new double[NC];
					}
					for (int i = 0; i < NB; i++)
						for (int j = 0; j < NC; j++)
							paramBC[0][i][j] = vparamBC[i][j];
					sgmA2 = new double[NSimu+1];
					sgmA2[0] = vsgmA2;
					sgmB2 = new double[NSimu+1];
					sgmB2[0] = vsgmB2;
					sgmC2 = new double[NSimu+1];
					sgmC2[0] = vsgmC2;
					sgmAB2 = new double[NSimu+1];
					sgmAB2[0] = vsgmAB2;
					sgmAC2 = new double[NSimu+1];
					sgmAC2[0] = vsgmAC2;
					sgmBC2 = new double[NSimu+1];
					sgmBC2[0] = vsgmBC2;
					sgmE2 = new double[NSimu+1];
					sgmE2[0] = vsgmE2;

					gen_SgmMu = vgen_SgmMu;
					gen_paramA = new double[NA];
					for (int t = 0; t < NA; t++)
						gen_paramA[t] = vgen_paramA[t];
					gen_paramB = new double[NB];
					for (int i = 0; i < NB; i++)
						gen_paramB[i] = vgen_paramB[i];
					gen_paramC = new double[NC];
					for (int i = 0; i < NC; i++)
						gen_paramC[i] = vgen_paramC[i];
					gen_paramAB = new double*[NA];
					for (int i = 0; i < NA; i++){
						gen_paramAB[i] = new double[NB];
						for (int j = 0; j < NB; j++)
							gen_paramAB[i][j] = vgen_paramAB[i][j];
					}
					gen_paramAC = new double*[NA];
					for (int i = 0; i < NA; i++){
						gen_paramAC[i] = new double[NC];
						for (int j = 0; j < NC; j++)
							gen_paramAC[i][j] = vgen_paramAC[i][j];
					}
					gen_paramBC = new double*[NB];
					for (int i = 0; i < NB; i++){
						gen_paramBC[i] = new double[NC];
						for (int j = 0; j < NC; j++)
							gen_paramBC[i][j] = vgen_paramBC[i][j];
					}
					gen_sgmA2 = vgen_sgmA2;
					gen_sgmB2 = vgen_sgmB2;
					gen_sgmC2 = vgen_sgmC2;
					gen_sgmAB2 = vgen_sgmAB2;
					gen_sgmAC2 = vgen_sgmAC2;
					gen_sgmBC2 = vgen_sgmBC2;
					gen_sgmE2 = vgen_sgmE2;

					cnt_mu = 0;
					cnt_paramA = new int[NA];
					for (int t = 0; t < NA; t++)
						cnt_paramA[t] = 0;
					cnt_paramB = new int[NB];
					for (int i = 0; i < NB; i++)
						cnt_paramB[i] = 0;
					cnt_paramC = new int[NC];
					for (int i = 0; i < NC; i++)
						cnt_paramC[i] = 0;
					cnt_paramAB = new int*[NA];
					for (int i = 0; i < NA; i++){
						cnt_paramAB[i] = new int[NB];
						for (int j = 0; j < NB; j++)
							cnt_paramAB[i][j] = 0;
					}
					cnt_paramAC = new int*[NA];
					for (int i = 0; i < NA; i++){
						cnt_paramAC[i] = new int[NC];
						for (int j = 0; j < NC; j++)
							cnt_paramAC[i][j] = 0;
					}
					cnt_paramBC = new int*[NB];
					for (int i = 0; i < NB; i++){
						cnt_paramBC[i] = new int[NC];
						for (int j = 0; j < NC; j++)
							cnt_paramBC[i][j] = 0;
					}
					cnt_sgmA2 = 0;
					cnt_sgmB2 = 0;
					cnt_sgmC2 = 0;
					cnt_sgmAB2 = 0;
					cnt_sgmAC2 = 0;
					cnt_sgmBC2 = 0;
					cnt_sgmE2 = 0;

					rho2_t = new double[NSimu+1];
					rho2_t[0] = sgmA2[0] / (sgmA2[0] + (sgmAB2[0] / NB) + (sgmAC2[0] / NC) + (sgmE2[0] / (NB * NC)));
	}

	void MCMC::do_mcmc(){
		DispMessage cfm;
		cfm.Show();
		cfm.SetMsg(MnMssg);
		for (int imcmc = 0; imcmc < NSimu; imcmc++){
			if((imcmc % 200) == 0){
				cfm.ShowMessage( "\n imcmc = " + imcmc.ToString() + "/" + NSimu.ToString());
			}

			double y = rn->normalMS( mu[imcmc], gen_SgmMu );

			double num = LogProb_mu(NA, NB, NC, XTable, y, paramA[imcmc], paramB[imcmc], paramC[imcmc],
				paramAB[imcmc], paramAC[imcmc], paramBC[imcmc],
				sgmA2[imcmc], sgmB2[imcmc], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
			double den = LogProb_mu(NA, NB, NC, XTable, mu[imcmc], paramA[imcmc], paramB[imcmc], paramC[imcmc],
				paramAB[imcmc], paramAC[imcmc], paramBC[imcmc],
				sgmA2[imcmc], sgmB2[imcmc], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
			double a = Math::Exp( num - den );
			if (rn->uni() < a){
				cnt_mu++;
				mu[imcmc+1] = y;
			}else{
				mu[imcmc+1] = mu[imcmc];
			}
			double * tA0 = new double[NA], * tA1 = new double[NA];
			for (int t = 0; t < NA; t++)
				tA0[t] = tA1[t] = paramA[imcmc][t];
			for (int t = 0; t < NA; t++){
				double y = rn->normalMS( tA0[t], gen_paramA[t] );
				tA1[t] = y;

				double num = LogProb_A(NA, NB, NC, XTable, mu[imcmc + 1], tA1, paramB[imcmc], paramC[imcmc],
					paramAB[imcmc], paramAC[imcmc], paramBC[imcmc],
					sgmA2[imcmc], sgmB2[imcmc], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
				double den = LogProb_A(NA, NB, NC, XTable, mu[imcmc + 1], tA0, paramB[imcmc], paramC[imcmc],
					paramAB[imcmc], paramAC[imcmc], paramBC[imcmc],
					sgmA2[imcmc], sgmB2[imcmc], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
				double a = Math::Exp( num - den );
				if (rn->uni() < a){
					cnt_paramA[t]++;
					tA0[t] = tA1[t];
				}else{
					tA1[t] = tA0[t];
				}
			}
			for (int t = 0; t < NA; t++)
				paramA[imcmc+1][t] = tA1[t];
			delete [] tA0;
			delete [] tA1;

			double * tB0 = new double[NB], * tB1 = new double[NB];
			for (int i = 0; i < NB; i++)
				tB0[i] = tB1[i] = paramB[imcmc][i];
			for (int i = 0; i < NB; i++){
				double y = rn->normalMS( tB0[i], gen_paramB[i] );
				tB1[i] = y;
				
				double num = LogProb_B(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], tB1, paramC[imcmc],
					paramAB[imcmc], paramAC[imcmc], paramBC[imcmc],
					sgmA2[imcmc], sgmB2[imcmc], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
				double den = LogProb_B(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], tB0, paramC[imcmc],
					paramAB[imcmc], paramAC[imcmc], paramBC[imcmc],
					sgmA2[imcmc], sgmB2[imcmc], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
				double a = Math::Exp( num - den );
				if (rn->uni() < a){
					cnt_paramB[i]++;
					tB0[i] = tB1[i];
				}else{
					tB1[i] = tB0[i];
				}
			}
			for (int i = 0; i < NB; i++)
				paramB[imcmc+1][i] = tB1[i];
			delete [] tB0;
			delete [] tB1;

			double * tC0 = new double[NC];
			double * tC1 = new double[NC];
			for (int i = 0; i < NC; i++)
				tC0[i] = tC1[i] = paramC[imcmc][i];
			for (int i = 0; i < NC; i++){
				double y = rn->normalMS( tC0[i], gen_paramC[i] );
				tC1[i] = y;

				double num = LogProb_C(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], tC1,
					paramAB[imcmc], paramAC[imcmc], paramBC[imcmc],
					sgmA2[imcmc], sgmB2[imcmc], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
				double den = LogProb_C(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], tC0,
					paramAB[imcmc], paramAC[imcmc], paramBC[imcmc],
					sgmA2[imcmc], sgmB2[imcmc], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
				double a = Math::Exp( num - den );
				if (rn->uni() < a){
					cnt_paramC[i]++;
					tC0[i] = tC1[i];
				}else{
					tC1[i] = tC0[i];
				}
			}
			for (int i = 0; i < NC; i++)
				paramC[imcmc+1][i] = tC1[i];
			delete [] tC0;
			delete [] tC1;
			
			double ** tAB0 = new double*[NA];
			for (int i = 0; i < NA; i++)
				tAB0[i] = new double[NB];
			double ** tAB1 = new double*[NA];
			for (int i = 0; i < NA; i++)
				tAB1[i] = new double[NB];
			for (int i = 0; i < NA; i++)
				for (int j = 0; j < NB; j++)
					tAB0[i][j] = tAB1[i][j] = paramAB[imcmc][i][j];
			for (int i = 0; i < NA; i++)
				for (int j = 0; j < NB; j++){
					double y = rn->normalMS( tAB0[i][j], gen_paramAB[i][j] );
					tAB1[i][j] = y;

					double num = LogProb_AB(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
						tAB1, paramAC[imcmc], paramBC[imcmc],
						sgmA2[imcmc], sgmB2[imcmc], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
					double den = LogProb_AB(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
						tAB0, paramAC[imcmc], paramBC[imcmc],
						sgmA2[imcmc], sgmB2[imcmc], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
					double a = Math::Exp( num - den );
					if (rn->uni() < a){
						cnt_paramAB[i][j]++;
						tAB0[i][j] = tAB1[i][j];
					}else{
						tAB1[i][j] = tAB0[i][j];
					}
				}
			for (int i = 0; i < NA; i++)
				for (int j = 0; j < NB; j++){
					paramAB[imcmc+1][i][j] = tAB1[i][j];
				}
			for (int i = 0; i < NA; i++){
				delete [] tAB0[i];
				delete [] tAB1[i];
			}
			delete [] tAB0;
			delete [] tAB1;
			
			double ** tAC0 = new double*[NA];
			for (int i = 0; i < NA; i++)
				tAC0[i] = new double[NC];
			double ** tAC1 = new double*[NA];
			for (int i = 0; i < NA; i++)
				tAC1[i] = new double[NC];
			for (int i = 0; i < NA; i++)
				for (int j = 0; j < NC; j++)
					tAC0[i][j] = tAC1[i][j] = paramAC[imcmc][i][j];
			for (int i = 0; i < NA; i++)
				for (int j = 0; j < NC; j++){
					double y = rn->normalMS( tAC0[i][j], gen_paramAC[i][j] );
					tAC1[i][j] = y;
					
					double num = LogProb_AC(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
						paramAB[imcmc + 1], tAC1, paramBC[imcmc],
						sgmA2[imcmc], sgmB2[imcmc], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
					double den = LogProb_AC(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
						paramAB[imcmc + 1], tAC0, paramBC[imcmc],
						sgmA2[imcmc], sgmB2[imcmc], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
					double a = Math::Exp( num - den );
					if (rn->uni() < a){
						cnt_paramAC[i][j]++;
						tAC0[i][j] = tAC1[i][j];
					}else{
						tAC1[i][j] = tAC0[i][j];
					}
				}
			for (int i = 0; i < NA; i++)
				for (int j = 0; j < NC; j++){
					paramAC[imcmc+1][i][j] = tAC1[i][j];
				}
			for (int i = 0; i < NA; i++){
				delete [] tAC0[i];
				delete [] tAC1[i];
			}
			delete [] tAC0;
			delete [] tAC1;
			
			double ** tBC0 = new double*[NB];
			for (int i = 0; i < NB; i++)
				tBC0[i] = new double[NC];
			double ** tBC1 = new double*[NB];
			for (int i = 0; i < NB; i++)
				tBC1[i] = new double[NC];
			for (int i = 0; i < NB; i++)
				for (int j = 0; j < NC; j++)
					tBC0[i][j] = tBC1[i][j] = paramBC[imcmc][i][j];
			for (int i = 0; i < NB; i++)
				for (int j = 0; j < NC; j++){
					double y = rn->normalMS( tBC0[i][j], gen_paramBC[i][j] );
					tBC1[i][j] = y;
					
					double num = LogProb_BC(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
						paramAB[imcmc + 1], paramAC[imcmc + 1], tBC1,
						sgmA2[imcmc], sgmB2[imcmc], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
					double den = LogProb_BC(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
						paramAB[imcmc + 1], paramAC[imcmc + 1], tBC0,
						sgmA2[imcmc], sgmB2[imcmc], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
					double a = Math::Exp( num - den );
					if (rn->uni() < a){
						cnt_paramBC[i][j]++;
						tBC0[i][j] = tBC1[i][j];
					}else{
						tBC1[i][j] = tBC0[i][j];
					}
				}
			for (int i = 0; i < NB; i++)
				for (int j = 0; j < NC; j++){
					paramBC[imcmc+1][i][j] = tBC1[i][j];
				}
			for (int i = 0; i < NB; i++){
				delete [] tBC0[i];
				delete [] tBC1[i];
			}
			delete [] tBC0;
			delete [] tBC1;


			y = rn->normalMS(sgmA2[imcmc], gen_sgmA2);
			if(y <= 0.0){
				a = -1.0;
			}else{
				num = LogProb_sgmA(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
					paramAB[imcmc + 1], paramAC[imcmc + 1], paramBC[imcmc + 1],
					y, sgmB2[imcmc], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
				den = LogProb_sgmA(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
					paramAB[imcmc + 1], paramAC[imcmc + 1], paramBC[imcmc + 1],
					sgmA2[imcmc], sgmB2[imcmc], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
				a = Math::Exp(num - den);
			}
			if(rn->uni() < a){
				cnt_sgmA2++;
				sgmA2[imcmc+1] = y;
			}else{
				sgmA2[imcmc+1] = sgmA2[imcmc];
			}

			y = rn->normalMS(sgmB2[imcmc], gen_sgmB2);
			if(y <= 0.0){
				a = -1.0;
			}else{
				num = LogProb_sgmB(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
					paramAB[imcmc + 1], paramAC[imcmc + 1], paramBC[imcmc + 1],
					sgmA2[imcmc], y, sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
				den = LogProb_sgmB(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
					paramAB[imcmc + 1], paramAC[imcmc + 1], paramBC[imcmc + 1],
					sgmA2[imcmc + 1], sgmB2[imcmc], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
				a = Math::Exp(num - den);
			}
			if(rn->uni() < a){
				cnt_sgmB2++;
				sgmB2[imcmc+1] = y;
			}else{
				sgmB2[imcmc+1] = sgmB2[imcmc];
			}
			
			y = rn->normalMS(sgmC2[imcmc], gen_sgmC2);
			if(y <= 0.0){
				a = -1.0;
			}else{
				num = LogProb_sgmC(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
					paramAB[imcmc + 1], paramAC[imcmc + 1], paramBC[imcmc + 1],
					sgmA2[imcmc], sgmB2[imcmc + 1], y, sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
				den = LogProb_sgmC(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
					paramAB[imcmc + 1], paramAC[imcmc + 1], paramBC[imcmc + 1],
					sgmA2[imcmc + 1], sgmB2[imcmc + 1], sgmC2[imcmc], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
				a = Math::Exp(num - den);
			}
			if(rn->uni() < a){
				cnt_sgmC2++;
				sgmC2[imcmc+1] = y;
			}else{
				sgmC2[imcmc+1] = sgmC2[imcmc];
			}

			y = rn->normalMS(sgmAB2[imcmc], gen_sgmAB2);
			if(y <= 0.0){
				a = -1.0;
			}else{
				num = LogProb_sgmAB(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
					paramAB[imcmc + 1], paramAC[imcmc + 1], paramBC[imcmc + 1],
					sgmA2[imcmc], sgmB2[imcmc + 1], sgmC2[imcmc + 1], y, sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
				den = LogProb_sgmAB(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
					paramAB[imcmc + 1], paramAC[imcmc + 1], paramBC[imcmc + 1],
					sgmA2[imcmc + 1], sgmB2[imcmc + 1], sgmC2[imcmc + 1], sgmAB2[imcmc], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
				a = Math::Exp(num - den);
			}
			if(rn->uni() < a){
				cnt_sgmAB2++;
				sgmAB2[imcmc+1] = y;
			}else{
				sgmAB2[imcmc+1] = sgmAB2[imcmc];
			}

			y = rn->normalMS(sgmAC2[imcmc], gen_sgmAC2);
			if(y <= 0.0){
				a = -1.0;
			}else{
				num = LogProb_sgmAC(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
					paramAB[imcmc + 1], paramAC[imcmc + 1], paramBC[imcmc + 1],
					sgmA2[imcmc], sgmB2[imcmc + 1], sgmC2[imcmc + 1], sgmAB2[imcmc + 1], y, sgmBC2[imcmc], sgmE2[imcmc]);
				den = LogProb_sgmAC(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
					paramAB[imcmc + 1], paramAC[imcmc + 1], paramBC[imcmc + 1],
					sgmA2[imcmc + 1], sgmB2[imcmc + 1], sgmC2[imcmc + 1], sgmAB2[imcmc + 1], sgmAC2[imcmc], sgmBC2[imcmc], sgmE2[imcmc]);
				a = Math::Exp(num - den);
			}
			if(rn->uni() < a){
				cnt_sgmAC2++;
				sgmAC2[imcmc+1] = y;
			}else{
				sgmAC2[imcmc+1] = sgmAC2[imcmc];
			}

			y = rn->normalMS(sgmBC2[imcmc], gen_sgmBC2);
			if(y <= 0.0){
				a = -1.0;
			}else{
				num = LogProb_sgmBC(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
					paramAB[imcmc + 1], paramAC[imcmc + 1], paramBC[imcmc + 1],
					sgmA2[imcmc], sgmB2[imcmc + 1], sgmC2[imcmc + 1], sgmAB2[imcmc + 1], sgmAC2[imcmc + 1], y, sgmE2[imcmc]);
				den = LogProb_sgmBC(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
					paramAB[imcmc + 1], paramAC[imcmc + 1], paramBC[imcmc + 1],
					sgmA2[imcmc + 1], sgmB2[imcmc + 1], sgmC2[imcmc + 1], sgmAB2[imcmc + 1], sgmAC2[imcmc + 1], sgmBC2[imcmc], sgmE2[imcmc]);
				a = Math::Exp(num - den);
			}
			if(rn->uni() < a){
				cnt_sgmBC2++;
				sgmBC2[imcmc+1] = y;
			}else{
				sgmBC2[imcmc+1] = sgmBC2[imcmc];
			}


			y = rn->normalMS(sgmE2[imcmc], gen_sgmE2);
			if(y <= 0.0){
				a = -1.0;
			}else{
				num = LogProb_sgmE(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
					paramAB[imcmc + 1], paramAC[imcmc + 1], paramBC[imcmc + 1],
					sgmA2[imcmc], sgmB2[imcmc + 1], sgmC2[imcmc + 1], sgmAB2[imcmc + 1], sgmAC2[imcmc + 1], sgmBC2[imcmc + 1], y);
				den = LogProb_sgmE(NA, NB, NC, XTable, mu[imcmc + 1], paramA[imcmc + 1], paramB[imcmc + 1], paramC[imcmc + 1],
					paramAB[imcmc + 1], paramAC[imcmc + 1], paramBC[imcmc + 1],
					sgmA2[imcmc + 1], sgmB2[imcmc + 1], sgmC2[imcmc + 1], sgmAB2[imcmc + 1], sgmAC2[imcmc + 1], sgmBC2[imcmc + 1], sgmE2[imcmc]);
				a = Math::Exp(num - den);
			}
			if(rn->uni() < a){
				cnt_sgmE2++;
				sgmE2[imcmc+1] = y;
			}else{
				sgmE2[imcmc+1] = sgmE2[imcmc];
			}

			rho2_t[imcmc+1] = sgmA2[imcmc+1] / (sgmA2[imcmc+1] + (sgmAB2[imcmc+1] / NB) + (sgmAC2[imcmc+1] / NC) + (sgmE2[imcmc+1] / (NB * NC)));
		}

		cfm.Close();
	}



	
	double g_a;

	double func1( double x ){
		double v = Math::Pow( x, g_a - 1.0 ) * Math::Exp( -x ); 
		return v;
	}

	double func2( double x ){
		double v = Math::Pow( -Math::Log( x ), g_a - 1.0 );
		return v;
	}


	double gamma( double a ){
			g_a = a;
			AdaptiveGL agl;
			double v = agl.integral( 0.0, 1.0, func1 ) + agl.integral( 0.0, Math::Exp(-1.0), func2 );
			return v;
	}


	double k_smoother( double x, double * samples, int N, double sgm ){
		double v = 0.0;
		for (int i = 0; i < N; i++)
			if (Math::Abs(x - samples[i]) < 4 * sgm){
				v += Math::Exp( -0.5 * sqr((x - samples[i]) / sgm));
			}
			return v / (N * sgm * Math::Sqrt(2.0 * Math::Atan(1.0) * 4.0));
	}


}
