#ifndef COMMON_CK
#define COMMON_CK

#include "RNP2to191.h"

using namespace System;
using namespace rnp2to191;

namespace cmmn {

	extern double *** XTable;
	extern double igm_aA, igm_bA, igm_aB, igm_bB, igm_aC, igm_bC,
					igm_aAB, igm_bAB, igm_aAC, igm_bAC, igm_aBC, igm_bBC, igm_aE, igm_bE;
	extern double * samples_sgmA2, * samples_sgmB2, * samples_sgmC2, 
					* samples_sgmAB2, * samples_sgmAC2, * samples_sgmBC2, * samples_sgmE2, * samples_Rho;
	extern double *** rho2t_nino, ** rho2t_LB, ** rho2t_UB, ** rho2t_Med;
	
	extern int N, NA, NB, NC;
	extern int MaxNi, MaxNo;

	inline double sqr( double v ){
			return  v * v;
	}

	ref class MCMC {
		public:
			String ^ MnMssg;
			int NSimu;
			int NA, NB, NC;
			double *** XTable;
			RNP2to191 * rn;
			double * mu, ** paramA, ** paramB, ** paramC, *** paramAB, *** paramAC, *** paramBC, 
					* sgmA2, * sgmB2, * sgmC2, * sgmAB2, * sgmAC2, * sgmBC2, * sgmE2;
			double gen_SgmMu, * gen_paramA, * gen_paramB, * gen_paramC,
					** gen_paramAB, ** gen_paramBC, ** gen_paramAC, gen_sgmA2, gen_sgmB2,
					gen_sgmC2, gen_sgmAB2, gen_sgmAC2, gen_sgmBC2, gen_sgmE2;
			int cnt_mu, * cnt_paramA, * cnt_paramB, * cnt_paramC, ** cnt_paramAB, ** cnt_paramAC, ** cnt_paramBC,
				cnt_sgmA2, cnt_sgmB2, cnt_sgmC2, cnt_sgmAB2, cnt_sgmAC2, cnt_sgmBC2, cnt_sgmE2;
			double * rho2_t;

			MCMC() { }

			void clearMe(){
				if (rn != nullptr)	
					delete rn;
				if (XTable != nullptr){
					for (int t = 0; t < NA; t++)
						delete [] XTable[t];
					delete [] XTable;
				}
				delete [] mu;
				if (paramA != nullptr){
					for (int j = 0; j <= NSimu; j++)
						delete [] paramA[j];
					delete [] paramA;
				}
				if (paramB != nullptr){
					for (int j = 0; j <= NSimu; j++)
						delete [] paramB[j];
					delete [] paramB;
				}
				if (paramC != nullptr){
					for (int i = 0; i <= NSimu; i++)
						delete [] paramC[i];
					delete [] paramC;
				}
				if (paramAB != nullptr){
					for (int i = 0; i <= NSimu; i++)
						delete [] paramAB[i];
					delete [] paramAB;
				}
				if (paramAC != nullptr){
					for (int i = 0; i <= NSimu; i++)
						delete [] paramAC[i];
					delete [] paramAC;
				}
				if (paramBC != nullptr){
					for (int i = 0; i <= NSimu; i++)
						delete [] paramBC[i];
					delete [] paramBC;
				}
				if (sgmA2 != nullptr) 
					delete [] sgmA2;
				if (sgmB2 != nullptr)
					delete [] sgmB2;
				if (sgmC2 != nullptr)
					delete [] sgmC2;
				if (sgmAB2 != nullptr)
					delete [] sgmAB2;
				if (sgmAC2 != nullptr)
					delete [] sgmAC2;
				if (sgmBC2 != nullptr)
					delete [] sgmBC2;
				if (sgmE2 != nullptr)
					delete [] sgmE2;

				if(gen_paramA != nullptr) delete [] gen_paramA;
				if(gen_paramB != nullptr) delete [] gen_paramB;
				if(gen_paramC != nullptr) delete [] gen_paramC;
				if(gen_paramAB != nullptr){
					for (int i = 0; i < NA; i++)
						delete [] gen_paramAB[i];
					delete [] gen_paramAB;
				}
				if(gen_paramAC != nullptr){
					for (int i = 0; i < NA; i++)
						delete [] gen_paramAC[i];
					delete [] gen_paramAC;
				}
				if(gen_paramBC != nullptr){
					for (int i = 0; i < NB; i++)
						delete [] gen_paramBC[i];
					delete [] gen_paramBC;
				}
				if(cnt_paramA != nullptr) delete [] cnt_paramA;
				if(cnt_paramB != nullptr) delete [] cnt_paramB;
				if(cnt_paramC != nullptr) delete [] cnt_paramC;
				if(cnt_paramAB != nullptr){
					for (int i = 0; i < NA; i++)
						delete [] cnt_paramAB[i];
					delete [] cnt_paramAB;
				}
				if(cnt_paramAC != nullptr){
					for (int i = 0; i < NA; i++)
						delete [] cnt_paramAC[i];
					delete [] cnt_paramAC;
				}
				if(cnt_paramBC != nullptr){
					for (int i = 0; i < NB; i++)
						delete [] cnt_paramBC[i];
					delete [] cnt_paramBC;
				}

				if(rho2_t != nullptr) delete [] rho2_t;
			}

			virtual ~MCMC(){ clearMe(); }
			!MCMC(){ clearMe(); }

			void Init(String ^ msg, int nJmpRN, int vNSimu, int vna, int vnb, int vnc,
						double *** vxtable, double vmu, double * vparamA, double * vparamB, double * vparamC,
						double ** vparamAB, double ** vparamAC, double ** vparamBC,
						double vsgmA2, double vsgmB2, double vsgmC2, double vsgmAB2, double vsgmAC2, double vsgmBC2,
						double vsgmE2, double vgen_SgmMu, double * vgen_paramA, double * vgen_paramB, double * vgen_paramC,
						double ** vgen_paramAB, double ** vgen_paramAC, double ** vgen_paramBC,
						double vgen_sgmA2, double vgen_sgmB2, double vgen_sgmC2, 
						double vgen_sgmAB2, double vgen_sgmAC2, double vgen_sgmBC2, double vgen_sgmE2);

			void do_mcmc();
	};

	double gamma( double a );

	double k_smoother( double x, double * samples, int N, double sgm );

}

#endif